#include <errno.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "log.h"
#include "util.h"

using namespace std;

int make_tempfile(const std::string &contents)
{
	char filename[] = "/tmp/cubemap.XXXXXX";
	int fd = mkstemp(filename);
	if (fd == -1) {
		perror("mkstemp");
		return -1;
	}

	if (unlink(filename) == -1) {
		perror("unlink");
		// Can still continue;
	}

	const char *ptr = contents.data();
	size_t to_write = contents.size();
	while (to_write > 0) {
		ssize_t ret = write(fd, ptr, to_write);
		if (ret == -1) {
			perror("write");
			close(fd);
			return -1;
		}

		ptr += ret;
		to_write -= ret;
	}

	return fd;
}

bool read_tempfile(int fd, std::string *contents)
{
	bool ok = true;
	ssize_t ret, has_read;

	off_t len = lseek(fd, 0, SEEK_END);
	if (len == -1) {
		perror("lseek");
		ok = false;
		goto done;
	}

	contents->resize(len);

	if (lseek(fd, 0, SEEK_SET) == -1) {
		perror("lseek");
		ok = false;
		goto done;
	}

	has_read = 0;
	while (has_read < len) {
		ret = read(fd, &((*contents)[has_read]), len - has_read);
		if (ret == -1) {
			perror("read");
			ok = false;
			goto done;
		}
		if (ret == 0) {
			log(ERROR, "Unexpected EOF!");
			ok = false;
			goto done;
		}
		has_read += ret;
	}

done:
	do {
		ret = close(fd);  // Implicitly deletes the files.
	} while (ret == -1 && errno == EINTR);
	
	if (ret == -1) {
		perror("close");
		// Can still continue.
	}

	return ok;
}
